<?php

namespace Drupal\postfile\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * File Controller class.
 */
class File extends ControllerBase {

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * Constructs a FileSystemForm object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   */
  public function __construct(RequestStack $request_stack, FileSystemInterface $file_system) {
    $this->requestStack = $request_stack;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function upload() {
    if (!$this->currentUser()->isAnonymous() && $this->currentUser()->hasPermission('postfile upload')) {
      /** @var \Symfony\Component\HttpFoundation\Request $request */
      $request = $this->requestStack->getCurrentRequest();
      /** @var \Drupal\Core\Config\Config $config */
      $config = $this->config('postfile.settings');
      /** @var \Symfony\Component\HttpFoundation\File\UploadedFile $uploaded_file */
      $uploaded_file = $request->files->get('file');

      // Prepare variables.
      $directory = $config->get('filesystem') . '://' . $config->get('directory');
      $file_name = basename($uploaded_file->getClientOriginalName());
      $destination = $directory . '/' . $file_name;

      // Handle the physical file.
      if ($this->fileSystem->prepareDirectory($directory, FileSystemInterface:: CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
        try {
          $this->fileSystem->copy($uploaded_file->getRealPath(), $destination, $config->get('exists_action'));

          /** @var \Drupal\file\Entity\File $file */
          $file = $this->entityTypeManager()
            ->getStorage('file')
            ->create(
              [
                'filename' => $file_name,
                'uri' => $destination,
                'status' => 1,
                'uid' => $this->currentUser()->id(),
              ]
            );
          $file->save();

          return new JsonResponse(['message' => 'Successfully uploaded the file - ' . $destination], 201);
        }
        catch (FileException $e) {
          return new JsonResponse(['message' => 'There was an error uploading the file - ' . $e->getMessage()], 500);
        }
      }
    }

    return new JsonResponse(['message' => 'There was an error preparing the directory for file upload.'], 500);
  }

}
